@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use 5.010;
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 # Session management for Hypersonic
 # Uses signed cookies for session ID, memory store for data
@@ -237,6 +237,58 @@ sub compile_session_ops {
       ->xs_return('1')
       ->xs_end;
 
+    # --------------------------------------------------------
+    # jit_session_get_set: Get or set session value
+    # Input: req (AV*), key (SV*), [value (SV*)]
+    # Output: value (SV*) or undef
+    # --------------------------------------------------------
+    $builder->xs_function('jit_session_get_set')
+      ->xs_preamble
+      ->line('if (items < 2) croak("Usage: _jit_session_get_set(req, key, [value])");')
+      ->line('SV** ary = AvARRAY((AV*)SvRV(ST(0)));')
+      ->line('SV* session_sv = ary[' . SLOT_SESSION . '];')
+      ->blank
+      ->comment('Check if session data exists')
+      ->if('!session_sv || !SvROK(session_sv) || SvTYPE(SvRV(session_sv)) != SVt_PVHV')
+        ->line('ST(0) = &PL_sv_undef;')
+        ->line('XSRETURN(1);')
+      ->endif
+      ->blank
+      ->line('HV* session = (HV*)SvRV(session_sv);')
+      ->line('STRLEN klen;')
+      ->line('const char* key = SvPV(ST(1), klen);')
+      ->blank
+      ->if('items == 2')
+        ->comment('Getter mode')
+        ->line('SV** val = hv_fetch(session, key, klen, 0);')
+        ->line('ST(0) = (val && *val) ? *val : &PL_sv_undef;')
+      ->else
+        ->comment('Setter mode')
+        ->line('hv_store(session, key, klen, newSVsv(ST(2)), 0);')
+        ->line('ary[' . SLOT_SESSION_MODIFIED . '] = newSViv(1);')
+        ->line('ST(0) = ST(2);')
+      ->endif
+      ->xs_return('1')
+      ->xs_end;
+
+    # --------------------------------------------------------
+    # jit_session_get_all: Get all session data
+    # Input: req (AV*)
+    # Output: session hashref (SV*) or undef
+    # --------------------------------------------------------
+    $builder->xs_function('jit_session_get_all')
+      ->xs_preamble
+      ->line('if (items < 1) croak("Usage: _jit_session_get_all(req)");')
+      ->line('SV** ary = AvARRAY((AV*)SvRV(ST(0)));')
+      ->line('SV* session_sv = ary[' . SLOT_SESSION . '];')
+      ->if('session_sv && SvROK(session_sv)')
+        ->line('ST(0) = session_sv;')
+      ->else
+        ->line('ST(0) = &PL_sv_undef;')
+      ->endif
+      ->xs_return('1')
+      ->xs_end;
+
     # Compile via XS::JIT
     eval {
         XS::JIT->compile(
@@ -250,6 +302,8 @@ sub compile_session_ops {
                 'Hypersonic::Session::_jit_constant_time_compare' => { source => 'jit_constant_time_compare', is_xs_native => 1 },
                 'Hypersonic::Session::_jit_generate_session_id'   => { source => 'jit_generate_session_id', is_xs_native => 1 },
                 'Hypersonic::Session::_jit_verify_signature'      => { source => 'jit_verify_signature', is_xs_native => 1 },
+                'Hypersonic::Session::_jit_session_get_set'       => { source => 'jit_session_get_set', is_xs_native => 1 },
+                'Hypersonic::Session::_jit_session_get_all'       => { source => 'jit_session_get_all', is_xs_native => 1 },
             },
         );
         $COMPILED = 1;
@@ -460,6 +514,14 @@ sub after_middleware {
 sub get_set {
     my ($req, $key, $value) = @_;
 
+    # Use JIT-compiled version if available
+    if ($COMPILED && defined &_jit_session_get_set) {
+        return @_ == 2
+            ? _jit_session_get_set($req, $key)
+            : _jit_session_get_set($req, $key, $value);
+    }
+
+    # Perl fallback
     my $data = $req->[SLOT_SESSION];
     return unless $data;
 
@@ -477,6 +539,12 @@ sub get_set {
 # Get all session data
 sub get_all {
     my ($req) = @_;
+
+    # Use JIT-compiled version if available
+    if ($COMPILED && defined &_jit_session_get_all) {
+        return _jit_session_get_all($req);
+    }
+
     return $req->[SLOT_SESSION];
 }
 
