@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use 5.010;
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 # JIT-compiled TLS/HTTPS support for Hypersonic
 # Uses OpenSSL for TLS support (via Alien::OpenSSL when available)
@@ -336,7 +336,46 @@ C
 
 # Generate SSL context initialization (for use by Hypersonic.pm code generation)
 sub gen_ssl_ctx_init {
-    return <<'C';
+    my (%opts) = @_;
+    my $enable_http2 = $opts{http2} // 0;
+    
+    my $alpn_code = '';
+    if ($enable_http2) {
+        $alpn_code = <<'ALPN';
+
+/* ALPN protocol list for HTTP/2 negotiation */
+static const unsigned char alpn_protos[] = {
+    2, 'h', '2',                              /* HTTP/2 */
+    8, 'h', 't', 't', 'p', '/', '1', '.', '1' /* HTTP/1.1 fallback */
+};
+
+/* ALPN selection callback - called during TLS handshake */
+static int alpn_select_cb(SSL* ssl,
+                          const unsigned char** out,
+                          unsigned char* outlen,
+                          const unsigned char* in,
+                          unsigned int inlen,
+                          void* arg) {
+    (void)ssl; (void)arg;
+    
+    /* Use OpenSSL's helper to select preferred protocol */
+    if (SSL_select_next_proto((unsigned char**)out, outlen,
+                              alpn_protos, sizeof(alpn_protos),
+                              in, inlen) == OPENSSL_NPN_NEGOTIATED) {
+        return SSL_TLSEXT_ERR_OK;
+    }
+    
+    /* No matching protocol - allow connection anyway (HTTP/1.1) */
+    return SSL_TLSEXT_ERR_NOACK;
+}
+ALPN
+    }
+    
+    my $alpn_setup = $enable_http2 
+        ? "\n    /* Enable ALPN for HTTP/2 negotiation */\n    SSL_CTX_set_alpn_select_cb(g_ssl_ctx, alpn_select_cb, NULL);\n" 
+        : '';
+    
+    return <<"C";
 /* Global SSL context - initialized once */
 static SSL_CTX* g_ssl_ctx = NULL;
 
@@ -346,11 +385,14 @@ typedef struct {
     SSL* ssl;
     time_t last_activity;
     int handshake_complete;
+    int protocol;  /* PROTO_HTTP1=1, PROTO_HTTP2=2 */
 } TLSConnection;
 
+#define PROTO_HTTP1 1
+#define PROTO_HTTP2 2
 #define MAX_TLS_CONNECTIONS 10000
 static TLSConnection g_tls_connections[MAX_TLS_CONNECTIONS];
-
+$alpn_code
 static inline TLSConnection* get_tls_connection(int fd) {
     for (int i = 0; i < MAX_TLS_CONNECTIONS; i++) {
         if (g_tls_connections[i].fd == fd) {
@@ -367,6 +409,7 @@ static inline TLSConnection* alloc_tls_connection(int fd, SSL* ssl) {
             g_tls_connections[i].ssl = ssl;
             g_tls_connections[i].last_activity = time(NULL);
             g_tls_connections[i].handshake_complete = 0;
+            g_tls_connections[i].protocol = PROTO_HTTP1;  /* Default */
             return &g_tls_connections[i];
         }
     }
@@ -387,6 +430,20 @@ static inline void free_tls_connection(int fd) {
     }
 }
 
+/* Check negotiated protocol after TLS handshake */
+static inline int get_negotiated_protocol(TLSConnection* conn) {
+    const unsigned char* alpn = NULL;
+    unsigned int alpn_len = 0;
+    SSL_get0_alpn_selected(conn->ssl, &alpn, &alpn_len);
+    
+    if (alpn_len == 2 && memcmp(alpn, "h2", 2) == 0) {
+        conn->protocol = PROTO_HTTP2;
+        return PROTO_HTTP2;
+    }
+    conn->protocol = PROTO_HTTP1;
+    return PROTO_HTTP1;
+}
+
 static int init_ssl_ctx(const char* cert_file, const char* key_file) {
     SSL_library_init();
     SSL_load_error_strings();
@@ -399,7 +456,7 @@ static int init_ssl_ctx(const char* cert_file, const char* key_file) {
 
     /* Set minimum TLS version to 1.2 for security */
     SSL_CTX_set_min_proto_version(g_ssl_ctx, TLS1_2_VERSION);
-
+$alpn_setup
     /* Load certificate and key */
     if (SSL_CTX_use_certificate_file(g_ssl_ctx, cert_file, SSL_FILETYPE_PEM) <= 0) {
         SSL_CTX_free(g_ssl_ctx);
